@@ -17,40 +17,15 @@
 local type = type
 local ipairs = ipairs
 local core = require("apisix.core")
-local utils = require("apisix.admin.utils")
+local resource = require("apisix.admin.resource")
 local get_routes = require("apisix.router").http_routes
 local get_services = require("apisix.http.service").services
 local compile_proto = require("apisix.plugins.grpc-transcode.proto").compile_proto
 local tostring = tostring
 
 
-local _M = {
-    version = 0.1,
-    need_v3_filter = true,
-}
-
-
-local function check_conf(id, conf, need_id)
-    if not conf then
-        return nil, {error_msg = "missing configurations"}
-    end
-
-    id = id or conf.id
-    if need_id and not id then
-        return nil, {error_msg = "missing proto id"}
-    end
-
-    if not need_id and id then
-        return nil, {error_msg = "wrong proto id, do not need it"}
-    end
-
-    if need_id and conf.id and tostring(conf.id) ~= tostring(id) then
-        return nil, {error_msg = "wrong proto id"}
-    end
-
-    core.log.info("schema: ", core.json.delay_encode(core.schema.proto))
-    core.log.info("conf  : ", core.json.delay_encode(conf))
-    local ok, err = core.schema.check(core.schema.proto, conf)
+local function check_conf(id, conf, need_id, schema)
+    local ok, err = core.schema.check(schema, conf)
     if not ok then
         return nil, {error_msg = "invalid configuration: " .. err}
     end
@@ -60,67 +35,10 @@ local function check_conf(id, conf, need_id)
         return nil, {error_msg = "invalid content: " .. err}
     end
 
-    return need_id and id or true
+    return true
 end
 
 
-function _M.put(id, conf)
-    local id, err = check_conf(id, conf, true)
-    if not id then
-        return 400, err
-    end
-
-    local key = "/protos/" .. id
-
-    local ok, err = utils.inject_conf_with_prev_conf("proto", key, conf)
-    if not ok then
-        return 503, {error_msg = err}
-    end
-
-    local res, err = core.etcd.set(key, conf)
-    if not res then
-        core.log.error("failed to put proto[", key, "]: ", err)
-        return 503, {error_msg = err}
-    end
-
-    return res.status, res.body
-end
-
-
-function _M.get(id)
-    local key = "/protos"
-    if id then
-        key = key .. "/" .. id
-    end
-
-    local res, err = core.etcd.get(key, not id)
-    if not res then
-        core.log.error("failed to get proto[", key, "]: ", err)
-        return 503, {error_msg = err}
-    end
-
-    utils.fix_count(res.body, id)
-    return res.status, res.body
-end
-
-
-function _M.post(id, conf)
-    local id, err = check_conf(id, conf, false)
-    if not id then
-        return 400, err
-    end
-
-    local key = "/protos"
-    utils.inject_timestamp(conf)
-    local res, err = core.etcd.push(key, conf)
-    if not res then
-        core.log.error("failed to post proto[", key, "]: ", err)
-        return 503, {error_msg = err}
-    end
-
-    return res.status, res.body
-end
-
 local function check_proto_used(plugins, deleting, ptype, pid)
 
     --core.log.info("check_proto_used plugins: ", core.json.delay_encode(plugins, true))
@@ -140,10 +58,7 @@ local function check_proto_used(plugins, deleting, ptype, pid)
     return true
 end
 
-function _M.delete(id)
-    if not id then
-        return 400, {error_msg = "missing proto id"}
-    end
+local function delete_checker(id)
     core.log.info("proto delete: ", id)
 
     local routes, routes_ver = get_routes()
@@ -182,16 +97,15 @@ function _M.delete(id)
     end
     core.log.info("proto delete service ref check pass: ", id)
 
-    local key = "/protos/" .. id
-    -- core.log.info("key: ", key)
-    local res, err = core.etcd.delete(key)
-    if not res then
-        core.log.error("failed to delete proto[", key, "]: ", err)
-        return 503, {error_msg = err}
-    end
-
-    return res.status, res.body
+    return nil, nil
 end
 
 
-return _M
+return resource.new({
+    name = "protos",
+    kind = "proto",
+    schema = core.schema.proto,
+    checker = check_conf,
+    unsupported_methods = {"patch"},
+    delete_checker = delete_checker
+})
