@@ -16,7 +16,6 @@
 --
 
 local core            = require("apisix.core")
-local ngx             = ngx
 local tostring        = tostring
 local http            = require("resty.http")
 local log_util        = require("apisix.utils.log-util")
@@ -87,6 +86,7 @@ local schema = {
             type = "string",
             default = "apisix.apache.org%2Flogs"
         },
+        log_format = {type = "object"},
     },
     oneOf = {
         { required = { "auth_config" } },
@@ -95,6 +95,13 @@ local schema = {
     encrypt_fields = {"auth_config.private_key"},
 }
 
+local metadata_schema = {
+    type = "object",
+    properties = {
+        log_format = log_util.metadata_schema_log_format,
+    },
+}
+
 
 local function send_to_google(oauth, entries)
     local http_new = http.new()
@@ -163,32 +170,39 @@ end
 
 
 local function get_logger_entry(conf, ctx, oauth)
-    local entry = log_util.get_full_log(ngx, conf)
-    local google_entry = {
-        httpRequest = {
-            requestMethod = entry.request.method,
-            requestUrl = entry.request.url,
-            requestSize = entry.request.size,
-            status = entry.response.status,
-            responseSize = entry.response.size,
-            userAgent = entry.request.headers and entry.request.headers["user-agent"],
-            remoteIp = entry.client_ip,
-            serverIp = entry.upstream,
-            latency = tostring(core.string.format("%0.3f", entry.latency / 1000)) .. "s"
-        },
-        jsonPayload = {
-            route_id = entry.route_id,
-            service_id = entry.service_id,
-        },
-        labels = {
-            source = "apache-apisix-google-cloud-logging"
-        },
-        timestamp = log_util.get_rfc3339_zulu_timestamp(),
-        resource = conf.resource,
-        insertId = ctx.var.request_id,
-        logName = core.string.format("projects/%s/logs/%s", oauth.project_id,
-                                     conf.log_id)
+    local entry, customized = log_util.get_log_entry(plugin_name, conf, ctx)
+    local google_entry
+    if not customized then
+        google_entry = {
+            httpRequest = {
+                requestMethod = entry.request.method,
+                requestUrl = entry.request.url,
+                requestSize = entry.request.size,
+                status = entry.response.status,
+                responseSize = entry.response.size,
+                userAgent = entry.request.headers and entry.request.headers["user-agent"],
+                remoteIp = entry.client_ip,
+                serverIp = entry.upstream,
+                latency = tostring(core.string.format("%0.3f", entry.latency / 1000)) .. "s"
+            },
+            jsonPayload = {
+                route_id = entry.route_id,
+                service_id = entry.service_id,
+            },
+        }
+    else
+        google_entry = {
+            jsonPayload = entry,
+        }
+    end
+
+    google_entry.labels = {
+        source = "apache-apisix-google-cloud-logging"
     }
+    google_entry.timestamp = log_util.get_rfc3339_zulu_timestamp()
+    google_entry.resource = conf.resource
+    google_entry.insertId = ctx.var.request_id
+    google_entry.logName = core.string.format("projects/%s/logs/%s", oauth.project_id, conf.log_id)
 
     return google_entry
 end
@@ -198,11 +212,16 @@ local _M = {
     version = 0.1,
     priority = 407,
     name = plugin_name,
+    metadata_schema = metadata_schema,
     schema = batch_processor_manager:wrap_schema(schema),
 }
 
 
-function _M.check_schema(conf)
+function _M.check_schema(conf, schema_type)
+    if schema_type == core.schema.TYPE_METADATA then
+        return core.schema.check(metadata_schema, conf)
+    end
+
     return core.schema.check(schema, conf)
 end
 
