@@ -122,7 +122,7 @@ local function send_http_data(conf, log_message)
 
     local httpc_res, httpc_err = httpc:request({
         method = "POST",
-        path = #url_decoded.path ~= 0 and url_decoded.path or "/",
+        path = url_decoded.path,
         query = url_decoded.query,
         body = log_message,
         headers = {
